#ifndef USUARIO_H
#define USUARIO_H

#ifdef __WIN32__
 #include <windows.h>
 #include <winsock.h>
#else
 #include <unistd.h>
#endif
#include "eventos.h"

#define BUF_PROC 2048

//------------------------------------------------------------------------------
// Aceitar conexes
class TServMud : public TServidor
{
public:
    void sEvento(short EvSock, struct sockaddr_in * localSockStr);
};

//------------------------------------------------------------------------------
// Comunicao com o MUD
class TClienteMud : public TComunica,public TRelogio
{
public:
    TClienteMud();
    void EnvMens(const char * endMens);
private:
    void cEvento(const char * buffer, int tamanho);
    void rEvento();
    void MensMud();
    unsigned short pontRec;     // Nmero do byte que est lendo
    char bufRec[BUF_PROC+2];    // Contm a mensagem recebida
    char dadoRecebido;          // Para controle da mensagem recebida
                                // 0=receber normalmente
                                // 2=recebendo caracter ESC
                                // 10=ignorar prximo caracter se for 13
                                // 13=ignorar prximo caracter se for 10
    char bufTelnet[30];
    int  pontTelnet;

    char bufESC[30];            // Para interpretar seqncias de ESC
    unsigned short pontESC;
};

//------------------------------------------------------------------------------
// Processar conexo
class TUsuario : public TComunica,public TRelogio
{
public:
    TUsuario();
    ~TUsuario();
    static int TotalObj;        // Nmero de objetos TUsuario

    static TUsuario *Primeiro,*Ultimo;
    TUsuario *Antes,*Depois;

    char Nome[TAM_NOME+1];  // Apelido digitado pelo usurio

    char Proto; // Protocolo:
                // 0=conectar telnet/papovox  1=telnet  2=papovox
    char Opcoes;// Opes de comunicao:
                // Bit 0=0 se deve filtrar cores e caracteres especiais
                // Telnet:  bit 1: 0=voltar ltimas mensagens ao pressionar Enter
                // Papovox: bit 1: 0=80 colunas  1=240 colunas
    char Passo; // 0:   rEvento() Apaga objeto
                // 1:   rEvento() Avisa processo pai (conexo via telnet/papovox)
                // 2-8: Est enviando mensagem HTTP para entrar no bate-papo
                // 9:   Enviando pgina HTML via ProcPagina(), j enviou cabealho
                //      rEvento() Apaga objeto
                // 10 em diante:
                //      Avisou processo pai que objeto foi criado
                //      Deve avisar processo pai quando objeto for apagado
                // 10:  Enviando mensagem "Digite seu apelido"
                // 12:  Enviando mensagem vazia a cada 5 minutos sem enviar mensagem
                // 13:  Parou temporariamente de receber mensagem de bot
                //      rEvento() faz cSockRec=cSockEnv e Passo=12
                // 14:  Fechando conexo de bot em rEvento()

// Para receber mensagens
    unsigned short pontRec;     // Nmero do byte que est lendo
    char bufRec[BUF_PROC+2];    // Contm a mensagem recebida
    char dadoRecebido;          // Para controle da mensagem recebida
                                // Telnet:
                                    // 0=receber normalmente
                                    // 1=ignorar prximo caracter
                                    // 2=ignorar prximos 2 caracteres
                                    // 3=receber primeiro caracter aps IAC
                                    // 4=recebeu IAC SB: ignorar at receber 240 (SE)
                                    // 10=ignorar prximo caracter se for 13
                                    // 13=ignorar prximo caracter se for 10
                                // HTTP:
                                    // 0-9=recebendo mensagem
                                    // 10-29=fim da mensagem
                                    // 30=desconecta se receber alguma cosa

    bool ProcMens(char * mens);

// Envia uma mensagem respeitando o protocolo
// Retorna verdadeiro se conseguiu enviar
    bool EnvMens(const char * endMens);
    static void EnvTodos(const char * texto);
    void EnvInicio();

// Eventos
    void cEvento(const char * buffer, int tamanho);
    void rEvento();
};

#endif
